@@ -121,7 +121,7 @@ void power::print_power(const print_context & c, const char *powersymbol, const
 
 void power::do_print_dflt(const print_dflt & c, unsigned level) const
 {
-       if ((-exponent).is_equal(_ex1)) {
+       if ((-exponent).is_integer_one()) {
 		// inverses printed in a special way
  	        if (level >= 20) c.s << "(";
 		c.s << "1/";
@@ -160,7 +160,7 @@ void power::do_print_dflt(const print_dflt & c, unsigned level) const
 		basis.print(c, precedence());
 		if (exp_parenthesis)
 			c.s << ')';
-		if (!exponent.is_equal(_ex_1)) {
+		if (!(-exponent).is_integer_one()) {
     		        c.s << "^" << expstr;
 		}
 		if (precedence() <= level)
@@ -199,7 +199,7 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 		if (base_parenthesis)
 			c.s << "\\right)";
 
-		if (!exponent.is_equal(_ex_1)) {
+		if (!(-exponent).is_integer_one()) {
     		        c.s << "^{";
 			bool exp_parenthesis = is_a<power>(exponent);
 			if (exp_parenthesis)
@@ -320,7 +320,8 @@ bool power::info(unsigned inf) const
 		case info_flags::positive:
 			return basis.info(info_flags::positive) && exponent.info(info_flags::real);
 		case info_flags::nonnegative:
-			return basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even);
+			return (basis.info(info_flags::positive) && exponent.info(info_flags::real)) ||
+			       (basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even));
 		case info_flags::has_indices: {
 			if (flags & status_flags::has_indices)
 				return true;
@@ -519,14 +520,11 @@ ex power::eval(int level) const
 	if (eexponent.is_zero() && 
 		!(basis_is_numerical && 
 			num_basis->is_parent_pos_char())) {
-		if (ebasis.is_zero())
-			throw (std::domain_error("power::eval(): pow(0,0) is undefined"));
-		else
-			return _ex1;
+		return _ex1;
 	}
 	
 	// ^(x,1) -> x
-	if (eexponent.is_equal(_ex1))
+	if (eexponent.is_integer_one())
 		return ebasis;
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
@@ -540,7 +538,7 @@ ex power::eval(int level) const
 	}
 
 	// ^(1,x) -> 1
-	if (ebasis.is_equal(_ex1))
+	if (ebasis.is_integer_one())
 		return _ex1;
 
 	// power of a function calculated by separate rules defined for this function
@@ -552,7 +550,6 @@ ex power::eval(int level) const
 		return power(ebasis.op(0), ebasis.op(1) * eexponent);
 
 	if (exponent_is_numerical) {
-
 		// ^(c1,c2) -> c1^c2  (c1, c2 numeric(),
 		// except if c1,c2 are rational, but c1^c2 is not)
 		if (basis_is_numerical) {
@@ -639,40 +636,43 @@ ex power::eval(int level) const
 			}
 		}
 	
-		// ^(*(x,y,z),c1) -> *(x^c1,y^c1,z^c1) (c1 integer)
-		if (num_exponent->is_integer() && is_exactly_a<mul>(ebasis)) {
-			return expand_mul(ex_to<mul>(ebasis), *num_exponent, 0);
-		}
+		if (num_exponent->is_integer()) {
+                        
+                        // ^(*(x,y,z),c1) -> *(x^c1,y^c1,z^c1) (c1 integer)
+                        if (is_exactly_a<mul>(ebasis)) {
+                                return expand_mul(ex_to<mul>(ebasis), *num_exponent, 0);
+                        }
 
-		// (2*x + 6*y)^(-4) -> 1/16*(x + 3*y)^(-4)
-		if (num_exponent->is_integer() && is_exactly_a<add>(ebasis)) {
-			numeric icont = ebasis.integer_content();
-			const numeric lead_coeff = 
-				ex_to<numeric>(ex_to<add>(ebasis).\
-						lead_coeff()).div(icont);
+                        // (2*x + 6*y)^(-4) -> 1/16*(x + 3*y)^(-4)
+                        if (is_exactly_a<add>(ebasis)) {
+                                numeric icont = ebasis.integer_content();
+                                const numeric lead_coeff = 
+                                        ex_to<numeric>(ex_to<add>(ebasis).\
+                                                        lead_coeff()).div(icont);
 
-			const bool canonicalizable = lead_coeff.is_integer();
-			const bool unit_normal = lead_coeff.is_pos_integer();
-			if (canonicalizable && (! unit_normal))
-				icont = icont.mul(*_num_1_p);
-			
-			if (canonicalizable && (icont != *_num1_p)) {
-				const add& addref = ex_to<add>(ebasis);
-				add* addp = new add(addref);
-				addp->setflag(status_flags::dynallocated);
-				addp->clearflag(status_flags::hash_calculated);
-				addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
-				addp->seq_sorted.resize(0);
-				for (epvector::iterator i = addp->seq.begin(); i != addp->seq.end(); ++i)
-					i->coeff = ex_to<numeric>(i->coeff).div_dyn(icont);
+                                const bool canonicalizable = lead_coeff.is_integer();
+                                const bool unit_normal = lead_coeff.is_pos_integer();
+                                if (canonicalizable && (! unit_normal))
+                                        icont = icont.mul(*_num_1_p);
 
-				const numeric c = icont.power(*num_exponent);
-				if (likely(c != *_num1_p))
-					return (new mul(power(*addp, *num_exponent), c))->setflag(status_flags::dynallocated);
-				else
-					return power(*addp, *num_exponent);
-			}
-		}
+                                if (canonicalizable && (icont != *_num1_p)) {
+                                        const add& addref = ex_to<add>(ebasis);
+                                        add* addp = new add(addref);
+                                        addp->setflag(status_flags::dynallocated);
+                                        addp->clearflag(status_flags::hash_calculated);
+                                        addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
+                                        addp->seq_sorted.resize(0);
+                                        for (epvector::iterator i = addp->seq.begin(); i != addp->seq.end(); ++i)
+                                                i->coeff = ex_to<numeric>(i->coeff).div_dyn(icont);
+
+                                        const numeric c = icont.power(*num_exponent);
+                                        if (likely(c != *_num1_p))
+                                                return (new mul(power(*addp, *num_exponent), c))->setflag(status_flags::dynallocated);
+                                        else
+                                                return power(*addp, *num_exponent);
+                                }
+                        }
+                }
 
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;1),c2),c1^c2)  (c1, c2 numeric(), c1>0)
 		// ^(*(...,x;c1),c2) -> *(^(*(...,x;-1),c2),(-c1)^c2)  (c1, c2 numeric(), c1<0)
@@ -709,9 +709,9 @@ ex power::eval(int level) const
 		}
 
 		// ^(nc,c1) -> ncmul(nc,nc,...) (c1 positive integer, unless nc is a matrix)
-		if (num_exponent->is_pos_integer() &&
-		    ebasis.return_type() != return_types::commutative &&
-		    !is_a<matrix>(ebasis)) {
+		if (ebasis.return_type() != return_types::commutative &&
+                    num_exponent->is_pos_integer() &&
+                    !is_a<matrix>(ebasis)) {
 			return ncmul(exvector(num_exponent->to_int(), ebasis), true);
 		}
 	}
@@ -944,6 +944,51 @@ ex power::expand(unsigned options) const
 		return *this;
 	}
 
+	// (x*p)^c -> x^c * p^c, if p>0
+	// makes sense before expanding the basis
+	if (is_exactly_a<mul>(basis) && !basis.info(info_flags::indefinite)) {
+		const mul &m = ex_to<mul>(basis);
+		exvector prodseq;
+		epvector powseq;
+		prodseq.reserve(m.seq.size() + 1);
+		powseq.reserve(m.seq.size() + 1);
+		epvector::const_iterator last = m.seq.end();
+		epvector::const_iterator cit = m.seq.begin();
+		bool possign = true;
+
+		// search for positive/negative factors
+		while (cit!=last) {
+			ex e=m.recombine_pair_to_ex(*cit);
+			if (e.info(info_flags::positive))
+				prodseq.push_back(pow(e, exponent).expand(options));
+			else if (e.info(info_flags::negative)) {
+				prodseq.push_back(pow(-e, exponent).expand(options));
+				possign = !possign;
+			} else
+				powseq.push_back(*cit);
+			++cit;
+		}
+
+		// take care on the numeric coefficient
+		ex coeff=(possign? _ex1 : _ex_1);
+		if (m.overall_coeff.info(info_flags::positive) && m.overall_coeff != _ex1)
+			prodseq.push_back(power(m.overall_coeff, exponent));
+		else if (m.overall_coeff.info(info_flags::negative) && m.overall_coeff != _ex_1)
+			prodseq.push_back(power(-m.overall_coeff, exponent));
+		else
+			coeff *= m.overall_coeff;
+
+		// If positive/negative factors are found, then extract them.
+		// In either case we set a flag to avoid the second run on a part
+		// which does not have positive/negative terms.
+		if (prodseq.size() > 0) {
+			ex newbasis = coeff*mul(powseq);
+			ex_to<basic>(newbasis).setflag(status_flags::purely_indefinite);
+			return ((new mul(prodseq))->setflag(status_flags::dynallocated)*(new power(newbasis, exponent))->setflag(status_flags::dynallocated).expand(options)).expand(options);
+		} else
+			ex_to<basic>(basis).setflag(status_flags::purely_indefinite);
+	}
+
 	const ex expanded_basis = basis.expand(options);
 	const ex expanded_exponent = exponent.expand(options);
 	
@@ -1127,11 +1172,11 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		
 		if (c.is_equal(_ex1)) {
 			if (is_exactly_a<mul>(r)) {
-				sum.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
-				                     _ex1));
+				sum.push_back(a.combine_ex_with_coeff_to_pair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
+				                                              _ex1));
 			} else {
-				sum.push_back(expair((new power(r,_ex2))->setflag(status_flags::dynallocated),
-				                     _ex1));
+				sum.push_back(a.combine_ex_with_coeff_to_pair((new power(r,_ex2))->setflag(status_flags::dynallocated),
+				                                              _ex1));
 			}
 		} else {
 			if (is_exactly_a<mul>(r)) {
